q = require 'q'
ask = require 'promptly'
request = require 'request'
colors = require 'colors'
Spinner = require('cli-spinner').Spinner

config = require('../config.coffee').data
module.exports = login = (program) ->
    program
        .command 'login'
        .description 'Login to the CEX registry'
        .action login


login = (options) ->
    spinner = new Spinner('processing...')
    if  config._auth?.user
        console.log "you are already logged in as #{config._auth.user.cyan}! \nyou can use #{'cex logout'.cyan} to log out"
        return
    oldCredentials = config._auth || {}
    oldEmail = config.email || ''
    promptsPromise = q.defer()
    askForUsername().then (username) ->
        askForPassword().then (password) ->
            if config.email
                promptsPromise.resolve [username, password, config.email]
                return

            askForEmail().then (email) ->
                promptsPromise.resolve([username, password, email])

    promptsPromise.promise.then ([username, password, email]) ->
        config._auth =
            user: username
            pass: password
        config.email = email
        spinner.start()
        user =
            name: username
            password: password
            email: email
            _id: 'org.couchdb.user:' + username
            type: 'user'
            roles: []
            date: new Date().toISOString()

        url = config.paths.login + encodeURIComponent(username)
        req = request.put url, (err, response, data) ->
            if response.statusCode is 201
                spinner.stop()
                console.log "User #{username.cyan} created successfully"
                return

            else if response.statusCode is 409
                request.get url + '?write=true', (err, response, body) ->
                    data = JSON.parse body
                    if err or data.error
                        console.log "Error authenticating user #{username.cyan}, please make sure your credentials are correct"
                        config._auth = oldCredentials
                        config.email = oldEmail
                        return

                    Object.keys(data).forEach (key) ->
                        if !user[key] || key is 'roles'
                            user[key] = data[key]
                    req = request.put url + '/-rev/' + user._rev, {
                        auth:
                            user: username
                            pass: password
                    }, (err, response, body) ->
                        data = JSON.parse body
                        if err or data.error
                            spinner.stop()
                            config._auth = oldCredentials
                            config.email = oldEmail
                            console.log "Error authenticating user #{username.cyan}, please make sure your credentials are correct"
                            return

                        spinner.stop()
                        console.log "User #{username.cyan} authenticated successfully"
                    req.write JSON.stringify(user)
            else
                spinner.stop()
                config._auth = oldCredentials
                config.email = oldEmail
                console.log "Error creating #{username.cyan}. Please try again"
        req.write(JSON.stringify(user))


askForUsername = ->
    usernamePromise = q.defer()

    ask.prompt 'Username:', {trim: true}, (err, value) ->
        if err
            usernamePromise.reject(err)
            return

        usernamePromise.resolve(value)

    return usernamePromise.promise

askForPassword = ->
    passwordPromise = q.defer()

    ask.password 'Password:', {trim: true}, (err, value) ->
        if err
            passwordPromise.reject(err)
            return

        passwordPromise.resolve(value)

    return passwordPromise.promise

askForEmail = ->
    emailPromise = q.defer()

    ask.prompt 'Email:', {trim: true}, (err, value) ->
        if err
            emailPromise.reject(err)
            return

        emailPromise.resolve(value)

    return emailPromise.promise
